import logging, argparse, ConfigParser, cookielib, os, pkgutil, inspect
import importlib

from . import settings
from . import commands
from .commands import Command


def parseConfig():
    cfg = ConfigParser.ConfigParser()
    cfg.read([settings.CONFIG_FILE_NAME])
    for (k, v) in cfg.items('user'):
        logging.info('loaded config: %s=%s' % (k, v))
        setattr(settings, k, v)

    try:
        cj = cookielib.MozillaCookieJar(settings.COOKIE_FILE_NAME)
        cj.load()
        cj.clear_expired_cookies()
        settings.cj = cj
    except:
        settings.cj = cookielib.MozillaCookieJar()
        logging.info('don\'t have existing cookie')


def _getOptionsParser():
    command_classes = find_cmd_classes(commands)

    _commands = [CC() for CC in command_classes]
    parser = argparse.ArgumentParser(prog='spoj', description='command line \
    tool for spoj.com')
    sub_parsers = parser.add_subparsers()

    for c in _commands:
        _parser = sub_parsers.add_parser(c.name, help=c.desc)
        c.add_arguments(_parser)

        _parser.set_defaults(func=c.do)

    return parser


def find_cmd_classes(pkg):
    path = os.path.dirname(pkg.__file__)
    pkgs = [name for _, name, _ in\
        pkgutil.iter_modules([path])]
    cmd = []

    for p in pkgs:
        p_name = '.'.join([pkg.__name__, p])
        ppg = importlib.import_module(p_name)
        classes = inspect.getmembers(ppg, inspect.isclass)
        for n, c in classes:
            if Command in c.__bases__:
                cmd.append(c)

    return cmd


def runner():
    parseConfig()
    parser = _getOptionsParser()
    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    runner()



import os

SPOJ_URL = 'http://www.spoj.com/'
LOGIN_URL = SPOJ_URL
spoj_dirname = 'ABRAMOV'
compiler_id = 'None'

ROOM_URL = lambda : SPOJ_URL + spoj_dirname +'/'

CONFIG_FILE_NAME = os.path.expanduser('~/.spojrc')
COOKIE_FILE_NAME = os.path.expanduser('~/.spoj_cookie')

user_name = None
#cookie jar
cj = None

_url = lambda path: ROOM_URL() + path + '/'



import htmlentitydefs, re, pydoc


def unescape(text):
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)


def text_table(soup_table, table_printer, skip_row=0,
        cell_formatter=lambda c: unescape(c.text)):
    t = table_printer

    rows = soup_table.findAll('tr')[skip_row:]
    for r in xrange(len(rows)):
        cells = rows[r].findAll('td')
        row = []
        for c in xrange(len(cells)):
            row.append(cell_formatter(r, c, cells[c]))

        t.add_row(row)

    return t


def pager(text):
    pydoc.pager(text.encode('utf-8'))



# -*- coding: utf-8 -*-
from . import Command
from ..settings import _url
from ..utils import unescape as _, pager


class ProblemDesc(Command):
    def __init__(self):
        super(ProblemDesc, self).__init__('desc',
                'describe a problem')

    def add_arguments(self, parser):
        parser.add_argument('problem_id')
        parser.add_argument('--input', action='store_true',
                help='only give example input')
        parser.add_argument('--output', action='store_true',
                help='only give example output')

    def doing(self, args):
        __, soup = self.get_soup(_url('problems/'+args.problem_id))

        title = '%s <%s>' % (_(soup.findAll('h1')[1].text), args.problem_id)
        pp = soup.findAll('p')
        desc = _(pp[1].text)
        if desc.endswith('Input'):
            desc = desc[:-5]
        inp = _(pp[2].text)
        if inp.endswith('Output'):
            inp = inp[:-6]
        out = _(pp[3].text)
        idx = out.find('ExampleInput:')
        if idx:
            out = out[:idx]

        example = _(soup.find('pre').text)
        idx = example.find('Output:')
        _in = example[6:idx]
        _out = example[idx+7:]

        content = '\n** %s **\n%s\n\n'
        content += 'input:\n%s\n---------------\n%s\n\n'
        content += 'output:\n%s\n---------------\n%s'
        content = content % (title, desc, inp, _in, out, _out)

        if args.input:
            content = _in
        if args.output:
            content =  _out

        pager(content)



# -*- coding: utf-8 -*-
from . import Command
from ..settings import _url
from ..utils import unescape as _


class UserStatus(Command):

    def __init__(self):
        super(UserStatus, self).__init__('status',
                'show user status of current or specied',
                )

    def doing(self, args):
        if args.user_name:
            path = 'users/' + args.user_name
        else:
            path = 'myaccount'
            self.auth_if()

        __, soup = self.get_soup(_url(path))
        title = _(soup.find('h3').text)
        solved_list = filter(None, [a.text for a in
            soup.findAll('table')[4].findAll('a')])
        unsolved_list = filter(None, [a.text for a in
            soup.findAll('table')[5].findAll('a')])

        print '%s\n' % title
        print '%d бодлого бүрэн бодсон:' % len(solved_list)
        print ', '.join(solved_list)
        print '%d бодлого дутуу бодсон:' % len(unsolved_list)
        print ', '.join(unsolved_list)

    def add_arguments(self, parser):
        parser.add_argument('user_name', nargs='?')



from . import Command
from .. import settings


class Authenticate(Command):
    def __init__(self):
        super(Authenticate, self).__init__('login',
                'Authenticate user to the app')

    def doing(self, args):
        if args.user_name:
            settings.user_name = args.user_name

        self.authenticate()

    def add_arguments(self, parser):
        parser.add_argument('user_name', nargs='?')




import os
from setuptools import setup, find_packages


name = 'spoj'
version = '0.0.15'


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name = name,
    version = version,
    author = "Nyambayar Turbat",
    author_email = "t.nyambayar@gmail.com",
    description = ("command line tool for spoj.com "),
    license = "BSD",
    keywords = "spoj spoj.com algorithm",
    url = "http://github.com/nyamba/spoj",
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    zip_safe=False,
    long_description=read('README'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "Topic :: Education",
        "License :: OSI Approved :: BSD License",
        "Intended Audience :: Customer Service",
    ],
    install_requires=[
        'setuptools',
        'prettytable',
        'argparse>=1.2.1',
        'requests>=1.1.0',
        'BeautifulSoup>=3.2.1',
        ],
    entry_points={
        'console_scripts': [
            'spoj = spoj.main:runner',
            ]
        },
)



# -*- coding: utf-8 -*-
import requests, getpass, cookielib
from BeautifulSoup import BeautifulSoup
from requests.cookies import RequestsCookieJar
from .. import settings
from ..settings import _url


class Command(object):

    def __init__(self, name, desc):
        self.name = name
        self.desc = desc
        self._session = None

    def save_cookies(self):
        mcj = cookielib.MozillaCookieJar(settings.COOKIE_FILE_NAME)
        [mcj.set_cookie(c) for c in self.getHTTPClient().cookies]
        mcj.save()

    def do(self, args):
        self.doing(args)
        self.save_cookies()

    def doing(self, args):
        '''
        chilren should implement it
        '''
        raise NotImplemented()

    def getHTTPClient(self):
        if self._session:
            return self._session

        rcj = RequestsCookieJar()
        for c in settings.cj:
            rcj.set_cookie(c)

        self._session = requests.Session()
        self._session.cookies = rcj

        return self._session

    def is_authenticated(self):
        try:
            assert self.requests.cookies['autologin_hash'], 'hash'
            assert self.requests.cookies['autologin_login'], 'login'
            __, soup = self.get_soup(_url('myaccount'))
            assert soup.find('h3'), 'h3'
            return True
        except:
            return False

    def auth_if(self):
        if not self.is_authenticated():
            self.authenticate()

    def __getattribute__(self, name):
        if name == 'requests':
            return self.getHTTPClient()
        else:
            return super(Command, self).__getattribute__(name)

    def add_arguments(self, parser):
        pass

    def authenticate(self):
        loop = True
        name = settings.user_name or 'user'
        print 'Authenticating %s for %s' % (name, settings.SPOJ_URL)

        self.requests.cookies.clear()

        while loop:
            name = settings.user_name
            if name is None:
                if settings.user_name:
                    name = settings.user_name
                    print 'loaded user name from config, [%s]' % name
                else:
                    name = raw_input('user name:')

            passw = getpass.getpass()

            payload = dict(login_user=name, password=passw,
                    autologin=1, submit='Log In')

            r = self.post(settings.LOGIN_URL, data=payload)
            if self.is_authenticated():
                print 'Welcome %s to %s' % (name, settings.SPOJ_URL)
                loop = False
            else:
                print 'incorrect user name or password! Try again.'

    def get(self, url, **kwargs):
        return self.requests.get(url, **kwargs)

    def post(self, url, data=None, **kwargs):
        return self.requests.post(url, data, **kwargs)

    def get_soup(self, url, **kwargs):
        r = self.get(url, **kwargs)
        soup = BeautifulSoup(r.text)
        return r, soup



# -*- coding: utf-8 -*-
from prettytable import PrettyTable
from . import Command
from . import settings
from ..settings import _url
from ..utils import unescape as _, text_table, pager


class ProblemList(Command):
    def __init__(self):
        super(ProblemList, self).__init__('list',
                'list all problems on %s' % settings.ROOM_URL())

    def doing(self, args):
        arguments = []
        if args.sort:
            arguments.append('sort=%d' % args.sort)
        if args.page:
            arguments.append('start=%d' % args.page)

        url = _url('problems/main') + ','.join(arguments)

        __, soup = self.get_soup(url)
        headers = []
        headers.append('Solution id(5)')
        if self.is_authenticated():
            headers.append('Solved(4)')
        headers.append('Problem name(1)')
        headers.append('Problem id(2)')
        headers.append('Users(6)')
        headers.append('Success(7)')

        total_col = len(headers)
        def cell_formatter(r, c, data_soup):
            if total_col > 5 and c == 1:
                return '*' if data_soup.find('img') else ''
            else:
                return _(data_soup.text)

        t = PrettyTable(headers)
        t.align["Problem name(1)"] = 'l'
        t.padding_with = 1

        table = soup.findAll('table',{'class': 'problems'})[-1]
        t = text_table(table, t, 1, cell_formatter)
        pager(t.get_string())

    def add_arguments(self, parser):
        choices, cc = [], [1, 2, 4, 5, 6, 7]
        for c in cc:
            choices.append(c)
            choices.append(-c)
        parser.add_argument('--page', type=int, nargs='?',
                help='page number of problem list page')
        parser.add_argument('--sort', type=int, choices=choices, nargs='?',
                help='''column number which will be sorted, if it is positive,
                ascending order else descending, available options:
                    1 - problem id,
                    2 - problem name,
                    5 - solution id,
                    6 - users count who solved it,
                    7 - percentage of valid solutions
                ''')



# -*- coding: utf-8 -*-
import sys, time
from . import Command
from . import settings
from ..settings import _url
from ..utils import unescape as _


LANG_COMP = {
        'py':    {4: 'python 2.7', 116: 'python 3.2.3'},
        'rb':    {17: 'ruby 1.9.3'},
        'php':   {29: 'php 5.2.6'},
        'java':  {10: 'java SE 6'},
        'jar':   {24: 'java SE 6'},
        'pas':   {22: 'fpc 2.2.4', 2: 'gpc 20070904'},
        'pl':    {3: 'perl 5.12.1'},
        'scala': {39: 'scala 2.8.0'},
        'sed':   {46: 'sed 4.2'},
        'c':     {11: 'gcc 4.3.2'},
        'cpp':   {41: 'g++ 4.3.2', 1: 'g++ 4.0.0-8'},
        'cs':    {27: 'gmcs 2.0.1'},
        'asm':   {13: 'nasm 2.03.01'},
        'go':    {114: 'gc 2010-07-14'},
        'sh':    {104: 'bash 4.0.37'},
        }


class TackleProblem(Command):
    def __init__(self):
        super(TackleProblem, self).__init__('tackle',
                'upload a solution to %s' % settings.ROOM_URL())
        self.wait_time = 5

    def add_arguments(self, parser):
        parser.add_argument('file_name', help='file_name must be formatted \
                like this: <problem_id>.<lang_extention>\n for example: \
                ABR0001.py')

    def doing(self, args):
        if settings.compiler_id:
            cmp_id = int(settings.compiler_id)
            print 'compiler id loaded from settings: %d' % cmp_id
        else:
            cmp_id, cmp_name = self.get_compiler(args.file_name)
            print 'Your solution will be compiled via %s(%d)' % (cmp_name,
            cmp_id)

        self.auth_if()
        problem_id = args.file_name.split('.')[0]
        url = _url('submit/complete')
        files = {'subm_file': (args.file_name, open(args.file_name, 'r'))}
        r = self.post(url,
                data={'lang':cmp_id,
                    'problemcode':problem_id,
                    'file': '',
                    'submit': 'submit',
                    },
                files=files)
        loop = True
        while loop:
            print 'waiting result for %d seconds!' % self.wait_time
            time.sleep(self.wait_time)
            print '\n **result** \n'
            result = self.get_result(problem_id)
            print 'date: %s\nname: %s\nstatus: %s\ntime: %s\nmemory: %s' %\
                    result
            if u'байна' not in result[2]:
                loop = False
            else:
                print 'fetching result again, ',


    def get_compiler(self, file_name):
        ext = file_name.split('.')[-1]
        compilers = LANG_COMP.get(ext, {})
        compiler_id = -1
        compiler_name = ''

        if len(compilers) == 0:
            raise ValueError('oops, don\'t have a compiler for your file')
        elif len(compilers) > 1:
            print 'there are many compilers,',
            while True:
                print 'please choose one of followings:'
                for k, v in compilers.iteritems():
                    print '%3d: %s' % (k, v)
                try:
                    compiler_id = input()
                    compiler_name = compilers[compiler_id]
                    break
                except:
                    print 'invalid!, only choose one id of provided'
        else:
            compiler_id, compiler_name = compilers.iteritems()[0]

        return compiler_id, compiler_name

    def get_result(self, problem_id):
        '''
        return last result of the problem
        '''
        url = _url('status/%s,%s' % (problem_id, settings.user_name))
        __, soup = self.get_soup(url)
        rows = soup.findAll('table', {'class': 'problems'})[1].\
                findAll('tr')[1].findAll('td')
        date = rows[2].text
        name = _(rows[3].text)
        status = _(rows[4].text)
        status = status.replace('edit', '')
        status = status.replace('run', '')
        time = rows[5].find('a').text
        mem = rows[6].text

        return (date, name, status, time, mem)
